@@ -1,5 +1,5 @@
 /* Return the next element of a path.
-   Copyright (C) 1992 Free Software Foundation, Inc.
+   Copyright (C) 1992, 2003, 2004, 2005 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
    USA.
 */
 
@@ -37,6 +37,9 @@
 #include <stdlib.h>
 #endif
 
+#include "nextelem.h"
+
+
 /* Return the next element of a colon-separated path.
    A null entry in the path is equivalent to "." (the current directory).
 
